@@ -9,12 +9,99 @@ use Math::Prime::Util::PP;
 package Math::Prime::Util;
 use Carp qw/carp croak confess/;
 
+BEGIN {
+  use constant CSPRNG_CHACHA => 1;
+  use constant CSPRNG_ISAAC  => 0;
+}
+
 *_validate_num = \&Math::Prime::Util::PP::_validate_num;
 *_validate_integer = \&Math::Prime::Util::PP::_validate_integer;
 *_prime_memfreeall = \&Math::Prime::Util::PP::_prime_memfreeall;
 *prime_memfree  = \&Math::Prime::Util::PP::prime_memfree;
 *prime_precalc  = \&Math::Prime::Util::PP::prime_precalc;
 
+if (CSPRNG_CHACHA) {
+  require Math::Prime::Util::ChaCha;
+  *_is_csprng_well_seeded = \&Math::Prime::Util::ChaCha::_is_csprng_well_seeded;
+  *_csrand = \&Math::Prime::Util::ChaCha::csrand;
+  *_srand = \&Math::Prime::Util::ChaCha::srand;
+  *random_bytes = \&Math::Prime::Util::ChaCha::random_bytes;
+  *irand = \&Math::Prime::Util::ChaCha::irand;
+  *irand64 = \&Math::Prime::Util::ChaCha::irand64;
+} elsif (CSPRNG_ISAAC) {
+  require Math::Prime::Util::ISAAC;
+  *_is_csprng_well_seeded = \&Math::Prime::Util::ISAAC::_is_csprng_well_seeded;
+  *_csrand = \&Math::Prime::Util::ISAAC::csrand;
+  *_srand = \&Math::Prime::Util::ISAAC::srand;
+  *random_bytes = \&Math::Prime::Util::ISAAC::random_bytes;
+  *irand = \&Math::Prime::Util::ISAAC::irand;
+  *irand64 = \&Math::Prime::Util::ISAAC::irand64;
+} else {
+  die "Bad CSPRNG choice";
+}
+
+# Fill all the mantissa bits for our NV, regardless of 32-bit or 64-bit Perl.
+{
+  use Config;
+  my $nvbits = (defined $Config{nvmantbits})  ? $Config{nvmantbits}
+             : (defined $Config{usequadmath}) ? 112
+             : 53;
+  my $uvbits = (~0 > 4294967295) ? 64 : 32;
+  my $rsub;
+  my $_tonv_32  = 1.0;        $_tonv_32 /= 2.0 for 1..32;
+  my $_tonv_64  = $_tonv_32;  $_tonv_64 /= 2.0 for 1..32;
+  my $_tonv_96  = $_tonv_64;  $_tonv_96 /= 2.0 for 1..32;
+  my $_tonv_128 = $_tonv_96;  $_tonv_128/= 2.0 for 1..32;
+  if ($uvbits == 64) {
+    if ($nvbits <= 32) {
+      *drand = sub { my $d = irand32() * $_tonv_32;  $d *= $_[0] if $_[0];  $d; };
+    } elsif ($nvbits <= 64) {
+      *drand = sub { my $d = irand64() * $_tonv_64;  $d *= $_[0] if $_[0];  $d; };
+    } else {
+      *drand = sub { my $d = irand64() * $_tonv_64 + irand64() * $_tonv_128;  $d *= $_[0] if $_[0];  $d; };
+    }
+  } else {
+    if ($nvbits <= 32) {
+      *drand = sub { my $d = irand() * $_tonv_32;  $d *= $_[0] if $_[0];  $d; };
+    } elsif ($nvbits <= 64) {
+      *drand = sub { my $d = ((irand() >> 5) * 67108864.0 + (irand() >> 6)) / 9007199254740992.0;  $d *= $_[0] if $_[0];  $d; };
+    } else {
+      *drand = sub { my $d = irand() * $_tonv_32 + irand() * $_tonv_64 + irand() * $_tonv_96 + irand() * $_tonv_128;  $d *= $_[0] if $_[0];  $d; };
+    }
+  }
+  *rand = \&drand;
+}
+
+
+*urandomb = \&Math::Prime::Util::PP::urandomb;
+*urandomm = \&Math::Prime::Util::PP::urandomm;
+
+# TODO: Go through these and decide if they should be doing anything extra here,
+#       such as input validation.
+# TODO: If not, why not the other functions?
+*sumdigits = \&Math::Prime::Util::PP::sumdigits;
+*todigits = \&Math::Prime::Util::PP::todigits;
+*todigitstring = \&Math::Prime::Util::PP::todigitstring;
+*fromdigits = \&Math::Prime::Util::PP::fromdigits;
+*inverse_li = \&Math::Prime::Util::PP::inverse_li;
+*sieve_prime_cluster = \&Math::Prime::Util::PP::sieve_prime_cluster;
+*twin_prime_count = \&Math::Prime::Util::PP::twin_prime_count;
+*ramanujan_prime_count = \&Math::Prime::Util::PP::ramanujan_prime_count;
+*sum_primes = \&Math::Prime::Util::PP::sum_primes;
+*print_primes = \&Math::Prime::Util::PP::print_primes;
+*sieve_range = \&Math::Prime::Util::PP::sieve_range;
+*is_carmichael = \&Math::Prime::Util::PP::is_carmichael;
+*is_quasi_carmichael = \&Math::Prime::Util::PP::is_quasi_carmichael;
+*is_pillai = \&Math::Prime::Util::PP::is_pillai;
+
+*random_prime = \&Math::Prime::Util::PP::random_prime;
+*random_ndigit_prime = \&Math::Prime::Util::PP::random_ndigit_prime;
+*random_nbit_prime = \&Math::Prime::Util::PP::random_nbit_prime;
+*random_proven_prime = \&Math::Prime::Util::PP::random_maurer_prime; # redir
+*random_strong_prime = \&Math::Prime::Util::PP::random_strong_prime;
+*random_maurer_prime = \&Math::Prime::Util::PP::random_maurer_prime;
+*random_shawe_taylor_prime =\&Math::Prime::Util::PP::random_shawe_taylor_prime;
+*miller_rabin_random = \&Math::Prime::Util::PP::miller_rabin_random;
 
 sub moebius {
   if (scalar @_ <= 1) {
@@ -108,10 +195,6 @@ sub nth_prime_upper {
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::nth_prime_upper($n);
 }
-sub inverse_li {
-  my($n) = @_;
-  return Math::Prime::Util::PP::inverse_li($n);
-}
 sub nth_prime_approx {
   my($n) = @_;
   _validate_positive_integer($n);
@@ -132,35 +215,26 @@ sub prime_count_approx {
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::prime_count_approx($n);
 }
-sub twin_prime_count {
-  my($low,$high) = @_;
-  return Math::Prime::Util::PP::twin_prime_count($low,$high);
-}
-sub ramanujan_prime_count {
-  my($low,$high) = @_;
-  return Math::Prime::Util::PP::ramanujan_prime_count($low,$high);
-}
-sub sum_primes {
-  my($low,$high) = @_;
-  return Math::Prime::Util::PP::sum_primes($low,$high);
-}
-sub print_primes {
-  my($low,$high,$fd) = @_;
-  return Math::Prime::Util::PP::print_primes($low,$high,$fd);
-}
-sub sieve_range {
-  my($n,$width,$depth) = @_;
-  return Math::Prime::Util::PP::sieve_range($n,$width,$depth);
-}
-sub sieve_prime_cluster {
-  my($low,$high,@cl) = @_;
-  return Math::Prime::Util::PP::sieve_prime_cluster($low,$high,@cl);
-}
 sub twin_prime_count_approx {
   my($n) = @_;
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::twin_prime_count_approx($n);
 }
+sub ramanujan_prime_count_lower {
+  my($n) = @_;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::ramanujan_prime_count_lower($n);
+}
+sub ramanujan_prime_count_upper {
+  my($n) = @_;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::ramanujan_prime_count_upper($n);
+}
+sub ramanujan_prime_count_approx {
+  my($n) = @_;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::ramanujan_prime_count_approx($n);
+}
 sub nth_twin_prime {
   my($n) = @_;
   _validate_positive_integer($n);
@@ -176,6 +250,21 @@ sub nth_ramanujan_prime {
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::nth_ramanujan_prime($n);
 }
+sub nth_ramanujan_prime_lower {
+  my($n) = @_;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::nth_ramanujan_prime_lower($n);
+}
+sub nth_ramanujan_prime_upper {
+  my($n) = @_;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::nth_ramanujan_prime_upper($n);
+}
+sub nth_ramanujan_prime_approx {
+  my($n) = @_;
+  _validate_positive_integer($n);
+  return Math::Prime::Util::PP::nth_ramanujan_prime_approx($n);
+}
 
 
 *is_prime          = \&Math::Prime::Util::PP::is_prime;
@@ -291,28 +380,14 @@ sub is_square_free {
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::is_square_free($n);
 }
-sub is_carmichael {
-  my($n) = @_;
-  #_validate_positive_integer($n);
-  return Math::Prime::Util::PP::is_carmichael($n);
-}
-sub is_quasi_carmichael {
-  my($n) = @_;
-  #_validate_positive_integer($n);
-  return Math::Prime::Util::PP::is_quasi_carmichael($n);
-}
 sub is_semiprime {
   my($n) = @_;
   _validate_positive_integer($n);
   return Math::Prime::Util::PP::is_semiprime($n);
 }
-sub is_pillai {
-  my($n) = @_;
-  #_validate_positive_integer($n);
-  return Math::Prime::Util::PP::is_pillai($n);
-}
 sub is_primitive_root {
   my($a,$n) = @_;
+  return 0 if $n == 0;
   $n = -$n if defined $n && $n < 0;
   $a %= $n if defined $a && $a < 0;
   _validate_positive_integer($a);
@@ -624,18 +699,6 @@ sub valuation {
   _validate_positive_integer($k);
   return Math::Prime::Util::PP::valuation($n, $k);
 }
-sub todigits {
-  my($n,$base,$len) = @_;
-  return Math::Prime::Util::PP::todigits($n,$base,$len);
-}
-sub todigitstring {
-  my($n,$base,$len) = @_;
-  return Math::Prime::Util::PP::todigitstring($n,$base,$len);
-}
-sub fromdigits {
-  my($n,$base) = @_;
-  return Math::Prime::Util::PP::fromdigits($n,$base);
-}
 sub hammingweight {
   my($n) = @_;
   $n = -$n if defined $n && $n < 0;
@@ -802,10 +865,6 @@ sub vecextract {
   return Math::Prime::Util::PP::vecextract(@_);
 }
 
-sub sumdigits {
-  return Math::Prime::Util::PP::sumdigits(@_);
-}
-
 1;
 
 __END__
