@@ -259,7 +259,7 @@ static UV count_segment_ranged(const unsigned char* sieve, UV nbytes, UV lowp, U
 
 #include "prime_count_tables.h"
 
-UV _XS_prime_count(UV low, UV high)
+UV segment_prime_count(UV low, UV high)
 {
   const unsigned char* cache_sieve;
   unsigned char* segment;
@@ -327,10 +327,30 @@ UV _XS_prime_count(UV low, UV high)
   return count;
 }
 
+UV prime_count(UV lo, UV hi)
+{
+  UV count, range_threshold;
+
+  if (lo > hi || hi < 2)
+    return 0;
+
+  /* We use table acceleration so this is preferable for small inputs */
+  if (hi < 66000000)  return segment_prime_count(lo, hi);
+
+  /* Rough empirical threshold for when segment faster than LMO */
+  range_threshold = hi / (isqrt(hi)/200);
+  if ( (hi-lo+1) < range_threshold )
+    return segment_prime_count(lo, hi);
+
+  return LMO_prime_count(hi) - ((lo < 2) ? 0 : LMO_prime_count(lo-1));
+
+  return count;
+}
+
 UV prime_count_approx(UV n)
 {
-  if (n < 3000000) return _XS_prime_count(2, n);
-  return (UV) (_XS_RiemannR( (long double) n ) + 0.5 );
+  if (n < 3000000) return segment_prime_count(2, n);
+  return (UV) (RiemannR( (long double) n ) + 0.5 );
 }
 
 /* See http://numbers.computation.free.fr/Constants/Primes/twin.pdf, page 5 */
@@ -345,7 +365,7 @@ UV twin_prime_count_approx(UV n)
     const long double two_over_log_two = 2.8853900817779268147198494L;
     long double ln = (long double) n;
     long double logn = logl(ln);
-    long double li2 = _XS_ExponentialIntegral(logn) + two_over_log_two-ln/logn;
+    long double li2 = Ei(logn) + two_over_log_two-ln/logn;
     /* try to minimize MSE */
     if (n < 32000000) {
       long double fm;
@@ -371,38 +391,28 @@ UV prime_count_lower(UV n)
 {
   long double fn, fl1, fl2, lower, a;
 
-  if (n < 33000) return _XS_prime_count(2, n);
+  if (n < 33000) return segment_prime_count(2, n);
 
   fn  = (long double) n;
   fl1 = logl(n);
   fl2 = fl1 * fl1;
 
-  if (n < UVCONST(52600000)) {
-#if 1
-    /* Dusart 2010, tweaked */
-    a = (n <    88783) ? 1.89L /* n >= 33000 */
-      : (n <   176000) ? 1.99L
-      : (n <   315000) ? 2.11L
-      : (n <  1100000) ? 2.19L
-      : (n <  4500000) ? 2.31L : 2.35L;
-    lower = fn/fl1 * (1.0L + 1.0L/fl1 + a/fl2);
-#else
-    /* Axler 2017, tweaked */
-    a = (n <    70870) ? -1.2 :
-        (n <   119620) ? -0.1 :
-        (n <   312230) ?  1.0 :
-        (n <   467530) ?  2.5 :
-        (n <  1090940) ?  3.0 :
-        (n <  1842470) ?  4.0 :
-        (n <  4413170) ?  5.0 :
-        (n <  6691000) ?  5.4 :
-        (n < 10384261) ?  5.8 : 6.0;
-    lower = fn/fl1 * (1.0L + 1.0L/fl1 + 2.0L/fl2 + a/(fl2*fl1));
-#endif
+  if (n < UVCONST(4000000000)) {
+    /* Loose enough that FP differences in Li(n) should be ok. */
+    a = (n <     88783) ?   4.0L
+      : (n <    300000) ?  -3.0L
+      : (n <    303000) ?   5.0L
+      : (n <   1100000) ?  -7.0L
+      : (n <   4500000) ? -37.0L
+      : (n <  10200000) ? -70.0L
+      : (n <  36900000) ? -53.0L
+      : (n <  38100000) ? -29.0L
+      :                   -84.0L;
+    lower = Li(fn) - (sqrtl(fn)/fl1) * (1.94L + 2.50L/fl1 + a/fl2);
   } else if (fn < 1e19) {          /* Büthe 2015 1.9      1511.02032v1.pdf */
-    lower = _XS_LogarithmicIntegral(fn) - (sqrtl(fn)/fl1) * (1.94L + 3.88L/fl1 + 27.57L/fl2);
+    lower = Li(fn) - (sqrtl(fn)/fl1) * (1.94L + 3.88L/fl1 + 27.57L/fl2);
   } else {                         /* Büthe 2014 v3 7.2   1410.7015v3.pdf */
-    lower = _XS_LogarithmicIntegral(fn) - fl1*sqrtl(fn)/25.132741228718345907701147L;
+    lower = Li(fn) - fl1*sqrtl(fn)/25.132741228718345907701147L;
   }
   return (UV) ceill(lower);
 }
@@ -442,7 +452,7 @@ UV prime_count_upper(UV n)
   int i;
   long double fn, fl1, fl2, upper, a;
 
-  if (n < 33000) return _XS_prime_count(2, n);
+  if (n < 33000) return segment_prime_count(2, n);
 
   fn  = (long double) n;
   fl1 = logl(n);
@@ -459,9 +469,9 @@ UV prime_count_upper(UV n)
       : (fn <  10010000000.0) ? 0.027    /* Empirical */
       : (fn < 101260000000.0) ? 0.021    /* Empirical */
                               : 0.0;
-    upper = _XS_LogarithmicIntegral(fn) - a * fl1*sqrtl(fn)/25.132741228718345907701147L;
+    upper = Li(fn) - a * fl1*sqrtl(fn)/25.132741228718345907701147L;
   } else {                       /* Büthe 2014 7.4 */
-    upper = _XS_LogarithmicIntegral(fn) + fl1*sqrtl(fn)/25.132741228718345907701147L;
+    upper = Li(fn) + fl1*sqrtl(fn)/25.132741228718345907701147L;
   }
   return (UV) floorl(upper);
 }
@@ -480,7 +490,7 @@ UV nth_prime_upper(UV n)
 
   if (n < 688383) {
     UV lo = fn * (flogn + flog2n - 1.0 + ((flog2n-2.10)/flogn));
-    UV hi = fn * (flogn + flog2n);
+    UV hi = fn * (flogn + flog2n - ((n <= 39017) ? 0 : 0.9484) );
     if (hi < lo) hi = MPU_MAX_PRIME;
     while (lo < hi) {
       UV mid = lo + (hi-lo)/2;
@@ -538,24 +548,7 @@ UV nth_prime_lower(UV n)
 
 UV nth_prime_approx(UV n)
 {
-  long double fn, flogn;
-  UV lo, hi;
-
-  if (n < NPRIMES_SMALL)
-    return primes_small[n];
-
-  /* Binary search for inverse Riemann R */
-  fn    = (long double) n;
-  flogn = logl(n);
-  lo    = (UV) (fn * flogn);
-  hi    = (UV) (fn * flogn * 2 + 2);
-  if (hi <= lo) hi = UV_MAX;
-  while (lo < hi) {
-    UV mid = lo + (hi-lo)/2;
-    if (_XS_RiemannR(mid) < fn) lo = mid+1;
-    else                        hi = mid;
-  }
-  return lo;
+  return (n < NPRIMES_SMALL)  ?  primes_small[n]  :  inverse_R(n);
 }
 
 
@@ -600,13 +593,13 @@ UV nth_prime(UV n)
     lower_limit += inverse_li(isqrt(n))/4;
     segment_size = lower_limit / 30;
     lower_limit = 30 * segment_size - 1;
-    count = _XS_LMO_pi(lower_limit);
+    count = prime_count(2,lower_limit);
 
     /* printf("We've estimated %lu too %s.\n", (count>n)?count-n:n-count, (count>n)?"FAR":"little"); */
-    /* printf("Our limit %lu %s a prime\n", lower_limit, _XS_is_prime(lower_limit) ? "is" : "is not"); */
+    /* printf("Our limit %lu %s a prime\n", lower_limit, is_prime(lower_limit) ? "is" : "is not"); */
 
     if (count >= n) { /* Too far.  Walk backwards */
-      if (_XS_is_prime(lower_limit)) count--;
+      if (is_prime(lower_limit)) count--;
       for (p = 0; p <= (count-n); p++)
         lower_limit = prev_prime(lower_limit);
       return lower_limit;
@@ -700,11 +693,11 @@ UV twin_prime_count(UV beg, UV end)
     end = (end-1) | 1;
     /* Cheesy way of counting the partial-byte edges */
     while ((beg % 30) != 1) {
-      if (_XS_is_prime(beg) && _XS_is_prime(beg+2) && beg <= end) sum++;
+      if (is_prime(beg) && is_prime(beg+2) && beg <= end) sum++;
       beg += 2;
     }
     while ((end % 30) != 29) {
-      if (_XS_is_prime(end) && _XS_is_prime(end+2) && beg <= end) sum++;
+      if (is_prime(end) && is_prime(end+2) && beg <= end) sum++;
       end -= 2;  if (beg > end) break;
     }
   }
@@ -725,7 +718,7 @@ UV twin_prime_count(UV beg, UV end)
       s = *sp;
       if (!(s & 0x0C)) sum++;
       if (!(s & 0x30)) sum++;
-      if (!(s & 0x80) && _XS_is_prime(seg_high+2)) sum++;
+      if (!(s & 0x80) && is_prime(seg_high+2)) sum++;
     }
     end_segment_primes(ctx);
   }
@@ -779,7 +772,7 @@ UV nth_twin_prime(UV n)
       for (p = 0; p < bytes; p++) {
         s >>= 8;
         if (p+1 < bytes)                    s |= (((UV)segment[p+1]) << 8);
-        else if (!_XS_is_prime(seg_high+2)) s |= 0xFF00;
+        else if (!is_prime(seg_high+2)) s |= 0xFF00;
         if (!(s & 0x000C) && !--n) { nth=seg_base+p*30+11; break; }
         if (!(s & 0x0030) && !--n) { nth=seg_base+p*30+17; break; }
         if (!(s & 0x0180) && !--n) { nth=seg_base+p*30+29; break; }
@@ -891,7 +884,7 @@ int sum_primes128(UV n, UV *hi_sum, UV *lo_sum) {
   New(0, V, r2+1, uint128_t);
   New(0, S, r2+1, uint128_t);
   for (k = 0; k <= r2; k++) {
-    UV v = (k <= r)  ?  k  :  n/(r2-k+1);
+    uint128_t v = (k <= r)  ?  k  :  n/(r2-k+1);
     V[k] = v;
     S[k] = (v*(v+1))/2 - 1;
   }
