@@ -8,7 +8,7 @@ extern void _XS_set_verbose(int v);
 extern int  _XS_get_callgmp(void);
 extern void _XS_set_callgmp(int v);
 
-extern int _XS_is_prime(UV x);
+extern int is_prime(UV x);
 extern UV  next_prime(UV x);
 extern UV  prev_prime(UV x);
 
@@ -27,12 +27,13 @@ extern UV*    _totient_range(UV low, UV high);
 extern IV     mertens(UV n);
 extern long double chebyshev_function(UV n, int which); /* 0 = theta, 1 = psi */
 
-extern long double _XS_ExponentialIntegral(long double x);
-extern long double _XS_LogarithmicIntegral(long double x);
+extern long double Ei(long double x);
+extern long double Li(long double x);
 extern long double ld_riemann_zeta(long double x);
-extern long double _XS_RiemannR(long double x);
+extern long double RiemannR(long double x);
 extern long double lambertw(long double k);
 extern UV inverse_li(UV x);
+extern UV inverse_R(UV x);
 
 extern int kronecker_uu(UV a, UV b);
 extern int kronecker_su(IV a, UV b);
